//this file is part of eMule
//Copyright (C)2002 Merkur ( merkur-@users.sourceforge.net / http://www.emule-project.net )
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#include "StdAfx.h"
#include "clientcredits.h"
#include "opcodes.h"
#include "emule.h"
#include "safefile.h"
#include <math.h>

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif


CClientCredits::CClientCredits(CFile& file){
	Load(file);
}

CClientCredits::CClientCredits(uchar* k){	
	uploaded = 0;
	downloaded = 0;
	lastseen = 0;
	reserved1 = 0;
	reserved2 = 0;
	reserved3 = 0;		
	md4cpy(key,k);	
}

CClientCredits::~CClientCredits(){	
}

void CClientCredits::AddDownloaded(uint32 bytes) {

	//encode
	uint64 current = reserved2;
	current = (current<<32) + downloaded + bytes ;

	//recode
	downloaded = (uint32)current;
	reserved2 = (uint32)(current>>32);

}

void CClientCredits::AddUploaded(uint32 bytes) {

	//encode
	uint64 current = reserved1;
	current = (current<<32) + uploaded + bytes ;

	//recode
	uploaded = (uint32)current;
	reserved1 = (uint32)(current>>32);
}

uint64	CClientCredits::GetUploadedTotal(){
	//encode
	//uint64 current=credits->reserved1;
	//current=(current<<32)+ credits->uploaded;

	return ( (uint64)reserved1<<32)+uploaded;
}

uint64	CClientCredits::GetDownloadedTotal(){
	//encode
	//uint64 current=credits->reserved2;
	//current=(current<<32)+ credits->downloaded ;
	return ( (uint64)reserved2<<32)+downloaded;
}

float CClientCredits::GetScoreRatio(bool inverted){
    EMULE_TRY
	// By Maverick
	// Added Inversion to calculate the clients ows ratio
	// at the remote client from ist own data	
	uint32 d = downloaded;
	uint32 u = uploaded;
	if (inverted)
	{
		d = uploaded;
		u = downloaded;
	}

	if (d < 1000000)
		return 1;
	float result = 0;
	if (u == 0)
		result = 10;
	else
		result = (float)(d*2)/u;
	float result2 = 0;
	result2 = (float)(d/1048576);
	result2 += 2;
	result2 = (double)sqrt((double)result2);

	if (result > result2)
		result = result2;

	if (result < 1)
		return 1;
	else if (result > 10)
		return 10;
	return result;
    EMULE_CATCH
    return 1;
}

bool CClientCredits::Load(CFile& file) {
	file.Read(key, sizeof(key));
	file.Read(&uploaded, sizeof(uploaded));
	file.Read(&downloaded, sizeof(downloaded));
	file.Read(&lastseen, sizeof(lastseen));
	file.Read(&reserved1, sizeof(reserved1));
	file.Read(&reserved2, sizeof(reserved2));
	file.Read(&reserved3, sizeof(reserved3));
	return true;
}

bool CClientCredits::Save(CFile& file) const {
	if (uploaded > 0 || downloaded > 0) { 
		file.Write(key, sizeof(key));
		file.Write(&uploaded, sizeof(uploaded));
		file.Write(&downloaded, sizeof(downloaded));
		file.Write(&lastseen, sizeof(lastseen));
		file.Write(&reserved1, sizeof(reserved1));
		file.Write(&reserved2, sizeof(reserved2));
		file.Write(&reserved3, sizeof(reserved3));
		return true;
	}
	return false;
}


CClientCreditsList::CClientCreditsList(CPreferences* in_prefs){
	app_prefs = in_prefs;
	lastsaved = ::GetTickCount();
	LoadList();
}

CClientCreditsList::~CClientCreditsList(){
	EMULE_TRY
	SaveList();
	CClientCredits* cur_credit;
	CCKey tmpkey(0);
	POSITION pos = client_map.GetStartPosition();
	while (pos){
		client_map.GetNextAssoc(pos,tmpkey,cur_credit);
		delete cur_credit;
	}
	client_map.RemoveAll();
	EMULE_CATCH
}

void CClientCreditsList::LoadList(){
	EMULE_TRY
	CSafeFile file;
	try{
		CString name = app_prefs->GetAppDir() + CString("clients.met");
		if (!file.Open(name,CFile::modeRead)){
			AddLogLine(false,IDS_ERR_LOADCREDITFILE);
			return;
		}
		uint8 version;
		file.Read(&version,1);
		if (version != CREDITFILE_VERSION){
			AddLogLine(false,IDS_ERR_CREDITFILEOLD);
			file.Close();
			return;
		}
		uint32 count;
		file.Read(&count,4);
		client_map.InitHashTable(count+5000);
		for (uint32 i = 0; i < count;i++){			
			CClientCredits* newcredits = new CClientCredits(file);
			client_map.SetAt(CCKey(newcredits->GetKey()),newcredits);
		}
		file.Close();
		AddLogLine(false,IDS_CREDITFILELOADED,count);
	}
	catch(CFileException* error){
		OUTPUT_DEBUG_TRACE();
		if (error->m_cause == CFileException::endOfFile)
			AddLogLine(true,IDS_CREDITFILECORRUPT);
		else{
			AddLogLine(true,IDS_ERR_CREDITFILEREAD,GetErrorMessage(error));
		}
		error->Delete();
	}
	EMULE_CATCH
}

void CClientCreditsList::SaveList(){
	EMULE_TRY
	CString name = app_prefs->GetAppDir() + CString("clients.met");
	CFile diskfile;
	CMemFile file(0, 0, 16384); 
	if (!diskfile.Open(name,CFile::modeWrite|CFile::modeCreate)){
		AddLogLine(true,IDS_ERR_FAILED_CREDITSAVE);
		return;
	}
	uint8 version = CREDITFILE_VERSION;
	file.Write(&version,1);
	uint32 count = client_map.GetCount();
	file.Write(&count,4);
	CClientCredits* cur_credit;
	CCKey tempkey(0);
	POSITION pos = client_map.GetStartPosition();
	count = 0;
	while (pos){
		client_map.GetNextAssoc(pos,tempkey,cur_credit);		
        if (cur_credit->Save(file))
			count++; 		
	}
    file.Seek(1,FILE_BEGIN); 
    file.Write(&count,4); 
    int size = file.GetLength(); 
    BYTE* tmp = file.Detach(); 
    diskfile.Write(tmp,size); 
    free(tmp); 
    diskfile.Close(); 
	lastsaved = ::GetTickCount();
	EMULE_CATCH
}

CClientCredits* CClientCreditsList::GetCredit(uchar* key){
	EMULE_TRY
	CClientCredits* result;
	CCKey tkey(key);
	if (!client_map.Lookup(tkey,result)){
		result = new CClientCredits(key);
		client_map.SetAt(CCKey(result->GetKey()),result);
	}
	result->SetLastSeen();
	return result;
	EMULE_CATCH
	return NULL;
}

void CClientCreditsList::Process(){
	if (::GetTickCount() - lastsaved > 70000)
		SaveList();
}