//this file is part of eMule
//Copyright (C)2002 Merkur ( merkur-@users.sourceforge.net / http://www.emule-project.net )
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#pragma once
#include "types.h"
#include "opcodes.h"
#include "otherfunctions.h"
#include "MD5Sum.h"
#include "securedvars.h"
#include "memcpy_amd.h"

const CString strDefaultToolbar = _T("0099010203040506079908");

#pragma pack(1)
// DO NOT EDIT VALUES like making a uint16 to uint32, or insert any value. ONLY append new vars
struct Preferences_Ext_Struct{
	int8	version;
	uchar	userhash[16];
	WINDOWPLACEMENT EmuleWindowPlacement;
};
// Maverick
struct ProxySettings{
	uint16 type;
	uint16 port;
	CString name;
	CString user;
	CString password;
	bool EnablePassword;
	bool UseProxy;
};

typedef enum
{
	SEE_SHARE_EVERYBODY = 0,
	SEE_SHARE_FRIENDS = 1,
	SEE_SHARE_NOONE = 2
} enSeeShare;

typedef enum
{
	HASH_PRIORITY_IDLE = 0,
	HASH_PRIORITY_LOWER = 1,
	HASH_PRIORITY_STANDARD = 2
} enHashPriority;

struct Preferences_Struct_DONTUSE{
	uchar	userhash[16];
	WINDOWPLACEMENT EmuleWindowPlacement;
	int16	downloadColumnWidths[14];
	BOOL	downloadColumnHidden[14];
	INT		downloadColumnOrder[14];
	int16	uploadColumnWidths[10];
	BOOL	uploadColumnHidden[10];
	INT		uploadColumnOrder[10];
	int16	queueColumnWidths[13];
	BOOL	queueColumnHidden[13];
	INT		queueColumnOrder[13];
	int16	searchColumnWidths[5];
	BOOL	searchColumnHidden[5];
	INT		searchColumnOrder[5];
	int16	sharedColumnWidths[12];	// netwolf: complete sources (zegzav) 13.05.03
	BOOL	sharedColumnHidden[12];
	INT		sharedColumnOrder[12];
	int16	serverColumnWidths[9];
	BOOL	serverColumnHidden[9];
	INT 	serverColumnOrder[9];
	int16	ircColumnWidths[3];
	BOOL	ircColumnHidden[3];
	INT		ircColumnOrder[3];

	uint8	flatbar;

	bool	BugReport;

	// Tarod
	uchar	A4AF_file_hash[16];

	int8	globalsearch; // Barry

	// Barry - Provide a mechanism for all tables to store/retrieve sort order
	int		tableSortItemDownload;
	int		tableSortItemUpload;
	int		tableSortItemQueue;
	int		tableSortItemSearch;
	int		tableSortItemShared;
	int		tableSortItemServer;
	bool	tableSortAscendingDownload;
	bool	tableSortAscendingUpload;
	bool	tableSortAscendingQueue;
	bool	tableSortAscendingSearch;
	bool	tableSortAscendingShared;
	bool	tableSortAscendingServer;
	CString	TxtEditor;
	bool	showRatesInTitle;
	CString	messageFilter;
	uint16	maxmsgsessions;

	char	m_sTemplateFile[MAX_PATH];	// netwolf: WS template path
	ProxySettings proxy; // deadlake PROXYSUPPORT
};

struct Preferences_Import19c_Struct{
	int8	version;
	TCHAR	nick[50];
	uint16	maxupload;
	uint16	maxdownload;
	uint16	port;
	uint16	maxconnections;
	int8	reconnect;
	int8	deadserver;
	int8	scorsystem;
	TCHAR	incomingdir[510];
	TCHAR	tempdir[510];
	int8	autoserverlist;
	int8	updatenotify;
	int8	mintotray;
	uchar	userhash[16];
	int8	autoconnect;
	int8	addServersFromConn;
};

struct Preferences_Import20a_Struct{
	int8	version;
	TCHAR	nick[50];
	uint16	maxupload;
	uint16	maxdownload;
	uint16	port;
	uint16	maxconnections;
	int8	reconnect;
	int8	deadserver;
	uint16	deadserverretries;
	int8	scorsystem;
	TCHAR	incomingdir[510];
	TCHAR	tempdir[510];
	int8	autoserverlist;
	int8	updatenotify;
	int8	mintotray;
	uchar	userhash[16];
	int8	autoconnect;
	int8	addServersFromConn;
	int8	addserversfromclient;
	int16	maxsourceperfile;
	int16	trafficOMeterInterval;
	int32   totalDownloaded;
	int32	totalUploaded;
	int	maxGraphDownloadRate;
	int	maxGraphUploadRate;
	uint8	beepOnError;
	uint8	confirmExit;
	WINDOWPLACEMENT EmuleWindowPlacement;
	int transferColumnWidths[9];
	int serverColumnWidths[8];
	uint8	splashscreen;
	uint8	filerBadIP;
};

struct Preferences_Import20b_Struct{
	int8	version;
	TCHAR	nick[50];
	uint16	maxupload;
	uint16	maxdownload;
	uint16	port;
	uint16	maxconnections;
	int8	reconnect;
	int8	deadserver;
	int8	scorsystem;
	TCHAR	incomingdir[510];
	TCHAR	tempdir[510];
	int8	autoserverlist;
	int8	updatenotify;
	int8	mintotray;
	uchar	userhash[16];
	int8	autoconnect;
	int8	addServersFromConn;
	int8	addserversfromclient;
	int16	maxsourceperfile;
	int16	trafficOMeterInterval;
	int32   totalDownloaded;	// outdated
	int32	totalUploaded;		// outdated
	int		maxGraphDownloadRate;
	int		maxGraphUploadRate;
	uint8	beepOnError;
	uint8	confirmExit;
	WINDOWPLACEMENT EmuleWindowPlacement;
	int transferColumnWidths[9];
	int serverColumnWidths[8];
	uint8	splashscreen;
	uint8	filerBadIP;
	int64   totalDownloadedBytes;
	int64	totalUploadedBytes;
};

class CPreferences{
public:
	enum Table {tableDownload, tableUpload, tableQueue, tableSearch, tableShared, tableServer, tableIrc};

	CPreferences();
	~CPreferences();

	CString	GetAppDir() const							{return appdir;}
	bool	Save();

	uint16	Profile()									{ return m_nProfile; }
	void	SetProfile(uint16 uProfile)					{ m_nProfile = uProfile; }
	int8	Score()										{ return m_nScoreSystem; }
	void	SetScore(int8 iScore)						{ m_nScoreSystem = iScore; }
	bool	Reconnect()									{ return m_bReconnect; }
	void	SetReconnect(bool bReconnect)				{ m_bReconnect = bReconnect; }
	int8	DeadServer()								{ return m_nDeadServer; }
	void	SetDeadServer(int8 iDead)					{ m_nDeadServer = iDead; }
	CString	GetUserNick()								{ return CString(m_sNick.Get()); }
	void    SetUserNick(const CString& strNewNick)		{ m_sNick.Put(strNewNick); }
	uint16	GetPort()									{ return m_nPort; }
	void	SetPort(uint16 uPort)						{ m_nPort = uPort; }
	
    //Maverick
    uint16	GetListenPort()								{ if(m_bUseProxyListenPort) return m_nListenPort; else return GetPort(); }
	void	SetListenPort(uint16 uPort)					{ m_nListenPort = uPort; m_bUseProxyListenPort = true; }
	void	ResetListenPort()							{ m_nListenPort = 0; m_bUseProxyListenPort = false; }
	
	uint16	GetUDPPort()								{ return m_nUDPPort; }
	void	SetUDPPort(uint16 uPort)					{ m_nUDPPort = uPort; }
	CString	GetIncomingDir()							{ return CString(m_sIncomingDir.Get()); }
	void	SetIncomingDir(const CString& strDir)		{ m_sIncomingDir.Put(strDir); }
	CString	GetTempDir()								{ return CString(m_sTempDir.Get()); }
	void	SetTempDir(const CString& strDir)			{ m_sTempDir.Put(strDir); }
	CString	GetVlcDir()  								{ return CString(m_sVlcDir.Get()); }		//Cax2 - keep for compatibility in 1a...
	CString	GetVideoPlayer()  							{ return CString(m_sVideoPlayer.Get()); }
	void    SetVideoPlayer(const CString& str)			{ m_sVideoPlayer.Put(str); }
	uchar*	GetUserHash()								{ return prefs->userhash; }
	uint16	GetMaxUpload()								{ return m_nMaxUpload; }

	// Added by Tarod
	bool	IsUAPEnabled()								{ return m_bUAP; }
	void	SetUAPEnabled(bool bEnable)					{ m_bUAP = bEnable; }
	bool	IsDAPEnabled()								{ return m_bDAP; }
	void	SetDAPEnabled(bool bEnable)					{ m_bDAP = bEnable; }
//	bool	ReconnectLowID() 							{ return prefs->reconnectlowid;}
	bool	IsDisabledXS()								{ return m_bDisableXS; }
	void	SetDisabledXS(bool bDisable)				{ m_bDisableXS = bDisable; }
	bool	StartDownloadPaused()						{ return m_bStartDownloadPaused; }
	void	SetStartDownloadPaused(bool bPaused)		{ m_bStartDownloadPaused = bPaused; }
	bool	DownloadPausedOnComplete()					{ return m_bDownloadPausedOnComplete; }
	void	SetDownloadPausedOnComplete(bool bPaused)	{ m_bDownloadPausedOnComplete = bPaused; }
	CString GetToolbarSettings() 						{ return CString(m_sToolbarSettings.Get()); }
	void	SetToolbarSettings(LPCTSTR in) 				{ m_sToolbarSettings.Put(in); }
	CString GetToolbarBitmapSettings() 					{ return CString(m_sToolbarBitmap.Get()); } // reCDVst
	void	SetToolbarBitmapSettings(const CString& path) { m_sToolbarBitmap.Put(path); } // reCDVst
	CString GetToolbarBitmapFolderSettings() 			{ return CString(m_sToolbarBitmapFolder.Get()); } // reCDVst
	void	SetToolbarBitmapFolderSettings(const CString& path){ m_sToolbarBitmapFolder.Put(path); } // reCDVst
	int8	GetToolbarLabelSettings()					{ return m_nToolbarLabels; }
	void	SetToolbarLabelSettings(int8 settings)		{ m_nToolbarLabels = settings; }
	bool	AutoServerlist()							{ return m_bAutoServerList; }
	void	SetAutoServerlist(bool bAuto)				{ m_bAutoServerList = bAuto; }
	uchar*	GetA4AFHash(void)							{ return prefs->A4AF_file_hash; }
#ifndef AMD
	void	SetA4AFHash(const uchar* file_hash)				{ memcpy(prefs->A4AF_file_hash, file_hash, sizeof(prefs->A4AF_file_hash)); }
	#else
	void	SetA4AFHash(const uchar* file_hash)				{ memcpy_amd(prefs->A4AF_file_hash, file_hash, sizeof(prefs->A4AF_file_hash)); }
	#endif

	// WebServer
	uint16	GetWSPort()									{ return m_nWebPort; }
	void	SetWSPort(uint16 uPort)						{ m_nWebPort = uPort; }
	CString	GetWSPass()									{ return CString(m_sWebPassword.Get()); }
	void	SetWSPass(const CString& strNewPass)				{ m_sWebPassword.Put(MD5Sum(strNewPass).GetHash()); }
	CString	GetWSLowPass()								{ return CString(m_sLowWebPassword.Get()); }
	void	SetWSLowPass(const CString& strNewPass)			{ m_sLowWebPassword.Put(MD5Sum(strNewPass).GetHash()); }
	bool	GetWSIsEnabled()							{ return m_bWebEnabled; }
	void	SetWSIsEnabled(bool bEnable)				{ m_bWebEnabled = bEnable; }
	bool	GetWSIsLowUserEnabled()						{ return m_bWebLowEnabled; }
	void	SetWSIsLowUserEnabled(bool bEnable)			{ m_bWebLowEnabled = bEnable; }
	bool	GetWebUseGzip()								{ return m_bWebUseGzip; }
	void	SetWebUseGzip(bool bUse)					{ m_bWebUseGzip = bUse; }
	int		GetWebPageRefresh()							{ return m_nWebPageRefresh; }
	void	SetWebPageRefresh(int nRefresh)				{ m_nWebPageRefresh = nRefresh; }

	bool	UpdateNotify()								{ return m_bUpdateNotify; }
	bool	DoMinToTray()								{ return m_bMinToTray; }
	void	SetMinToTray(bool bMinToTray)				{ m_bMinToTray = bMinToTray; }
	bool	DoAutoConnect() 							{ return m_bAutoConnect; }
	void    SetAutoConnect(bool bConnect)				{ m_bAutoConnect = bConnect; }
	uint32	BadClientBanTime()							{ return m_nBadClientBanTime; }
	void	SetBadClientBanTime(uint32 uTime)			{ m_nBadClientBanTime = uTime; }
	uint32	BadClientMinRequestTime()					{ return m_nMinRequestTime; }
	void	SetBadClientMinRequestTime(uint32 uTime)	{ m_nMinRequestTime = uTime; }
	uint8	BadClientMinRequestNum()					{ return m_nBadClientBan; }
	void	SetBadClientMinRequestNum(uint8 uVal)		{ m_nBadClientBan = uVal; }
	bool	IsBanMessageEnabled()						{ return m_bBanMessageEnabled; }
	void	SetBanMessageEnabled(bool bEnable)          { m_bBanMessageEnabled = bEnable; }
	// DbT:BanEnDisable
	bool	BanEnabled()								{ return m_bBanEnabled; }
	void	SetBanEnable(bool bEnable)					{ m_bBanEnabled = bEnable; }
	// DbT:End
	bool	GetDisableRatingIcons()						{ return m_bDisableRatingIcons; }
	void	SetDisableRatingIcons(bool bEnable)		    { m_bDisableRatingIcons = bEnable; }

	// reCDVst (Load/Save Sources settings)
	uint32	RestoreSourcesTime()						{ return m_nSLSRestoreSourcesTime; }
	void	SetRestoreSourcesTime(uint32 uTime)			{ m_nSLSRestoreSourcesTime = uTime; }
	uint32	WhenSourcesOutdated()						{ return m_nSLSOutdated; }
	void	SetWhenSourcesOutdated(uint32 uVal)			{ m_nSLSOutdated = uVal;}
	uint32	SLSMaxSourcesPerFile()						{ return m_SLSMaxSourcesPerFile; }
	void	SetSLSMaxSourcesPerFile(uint32 MaxSrc)		{ m_SLSMaxSourcesPerFile = MaxSrc; }
	bool	SLSEnable()									{ return m_bSLSEnable; }
	void	SetSLSEnable(bool bEnable)					{ m_bSLSEnable = bEnable; }

	int16	GetQueueSize()								{ return m_nQueueSize; }
	void	SetQueueSize(int16 iSize)					{ m_nQueueSize = iSize; }
	int16	GetRemoteQueueLimit()						{ return m_nRQlimit; }
	void	SetRemoteQueueLimit(uint16 uVal)			{ m_nRQlimit = uVal; }
	bool	UseRemoteQueueLimit()						{ return m_bUseRQlimit; }		//Cax2 23/02/03
	void	UseRemoteQueueLimit(bool bit)				{ m_bUseRQlimit = bit; }
	
	bool	GetAddServersFromConn()						{ return m_bAddServersFromConn; }
	void	SetAddServersFromConn(bool bAdd)			{ m_bAddServersFromConn = bAdd; }
	uint16	GetTrafficOMeterInterval() 					{ return m_nTrafficOMeterInterval; }
	void	SetTrafficOMeterInterval(uint16 in) 		{ m_nTrafficOMeterInterval = in; }
	uint16	GetStatsInterval() 							{ return m_nStatsInterval; }
	void	SetStatsInterval(uint16 in) 				{ m_nStatsInterval = in; }
	uint16  GetStatsColumnWidth()						{ return m_nStatsColumnWidth; }
	uint16  GetDetailColumnWidth()						{ return m_nDetailColumnWidth; }
	void    SetStatsColumnWidth(uint16 in)				{ m_nStatsColumnWidth = in; }
	void    SetDetailColumnWidth(uint16 in)				{ m_nDetailColumnWidth = in; }
	CString	GetStatsListExpanded()						{ return CString(m_sStatsListExpanded.Get()); }
	void	SetStatsListExpanded(const CString& in)		{ m_sStatsListExpanded.Put(in); }
	void	Add2TotalDownloaded(uint64 in)				{ uint64 nSet = m_nTotalDownloadedBytes.Get(); nSet += in; m_nTotalDownloadedBytes.Put(nSet); }
	uint64  GetTotalDownloaded()						{ return m_nTotalDownloadedBytes.Get(); }
	void	Add2TotalUploaded(uint64 in) 				{ uint64 nSet = m_nTotalUploadedBytes.Get(); nSet += in; m_nTotalUploadedBytes.Put(nSet); }
	uint64	GetTotalUploaded()							{ return m_nTotalUploadedBytes.Get(); }
	bool	IsErrorBeepEnabled()						{ return m_bBeepOnError; }
	void	SetErrorBeepEnabled(bool bEnable)			{ m_bBeepOnError = bEnable; }
	bool	IsConfirmExitEnabled()						{ return m_bConfirmExit; }
	void	SetConfirmExitEnabled(bool bEnable)			{ m_bConfirmExit = bEnable; }
	bool	UseSplashScreen()							{ return m_bSplashScreen; }
	void	SetUseSplashScreen(bool bUse)				{ m_bSplashScreen = bUse; }
	bool	FilterBadIPs()								{ return m_bFilterBadIP; }
	void    SetFilterBadIPs(bool bFilter)				{ m_bFilterBadIP = bFilter; }
	bool	FilterServersByIP()							{ return m_bFilterServersByIP; }
	void	SetFilterServersByIP (bool bFilter)			{ m_bFilterServersByIP = bFilter; }
	bool	IsOnlineSignatureEnabled()  				{ return m_bOnlineSig; }
	void	SetOnlineSignatureEnabled(bool bEnable)		{ m_bOnlineSig = bEnable; }

	int		GetMaxGraphUploadRate()						{ return m_nMaxGraphUploadRate; }
	void	SetMaxGraphUploadRate(int in)				{ m_nMaxGraphUploadRate = in; }
	int		GetMaxGraphDownloadRate()					{ return m_nMaxGraphDownloadRate; }
	void	SetMaxGraphDownloadRate(int in)				{ m_nMaxGraphDownloadRate = in; }

	uint16	GetMaxDownload();
	uint16	GetMaxConnections()							{ return m_nMaxConnections; }
	void	SetMaxConnections(uint16 uMaxConn)			{ m_nMaxConnections = uMaxConn; }
	uint16	GetMaxSourcePerFile()						{ return m_nMaxSourcePerFile; }
	void	SetMaxSourcePerFile(uint16 uVal)			{ m_nMaxSourcePerFile = uVal; }
	uint16	GetMaxSourcePerFileSoft()					{ uint16 temp = m_nMaxSourcePerFile*0.9;  return (temp > 1000)?1000:temp; }
	uint16	GetMaxSourcePerFileUDP()					{ uint16 temp = m_nMaxSourcePerFile*0.75; return (temp > 100)? 100:temp; }
	uint16	GetDeadserverRetries()						{ return m_nDeadServerRetries; }
	void	SetDeadserverRetries(uint16 uRetries)		{ m_nDeadServerRetries = uRetries; }

	int     GetColumnWidth (Table t, int index) const;
	BOOL    GetColumnHidden(Table t, int index) const;
	int     GetColumnOrder (Table t, int index) const;
	void	SetColumnWidth (Table t, int index, int width);
	void	SetColumnHidden(Table t, int index, BOOL bHidden);
	void	SetColumnOrder (Table t, INT *piOrder);

	// Barry - Provide a mechanism for all tables to store/retrieve sort order
	int		GetColumnSortItem (Table t) const;
	bool	GetColumnSortAscending (Table t) const;
	void	SetColumnSortItem (Table t, int sortItem);
	void	SetColumnSortAscending (Table t, bool sortAscending);



	bool	GetMultiple()								{ return m_bMultiple; }
	void	SetMultiple(bool bEnable)					{ m_bMultiple = bEnable; }

	WORD	GetLanguageID()								{ return m_nLanguageID; }
	void	SetLanguageID(WORD wID)						{ m_nLanguageID = wID; }
	int8	IsDoubleClickEnabled()						{ return m_nTransferDoubleclick; }
	void	SetDoubleClickEnabled(int8 iEnable)			{ m_nTransferDoubleclick = iEnable; }
	enSeeShare CanSeeShares(void)						{ return m_nSeeShares.Get(); }
	void	SetCanSeeShares(enSeeShare iCan)			{ m_nSeeShares.Put(iCan); }
	int8	GetToolTipDelay(void)						{ return m_nTooltipDelayTime; }
	void	SetToolTipDelay(int8 iDelay)				{ m_nTooltipDelayTime = iDelay; }
	bool	IsBringToFront()							{ return m_bBringToForeground; }
	void	SetBringToFront(bool bBTFront)				{ m_bBringToForeground = bBTFront; }

	int8    GetSplitterbarPosition()					{ return m_nSplitterBarPosition; }
	void	SetSplitterbarPosition(int8 pos) 			{ m_nSplitterBarPosition = pos; }
	uint16	GetStatsMax()								{ return m_nStatsMax; }
	void	SetStatsMax(uint16 uMaxStats)				{ m_nStatsMax = uMaxStats; }
	int8	GetStatsAverageMinutes()					{ return m_nStatsAverageMinutes; }
	void	SetStatsAverageMinutes(int8 in)				{ m_nStatsAverageMinutes = in; }

	bool	GetShowToolbarSpeedMeter()					{ return m_bShowToolbarSpeedMeter; }
	void	SetShowToolbarSpeedMeter(bool bShow)		{ m_bShowToolbarSpeedMeter = bShow; }
	bool    GetCloseToTray()							{ return m_bCloseToTray; }
	void	SetCloseToTray(bool bClose)					{ m_bCloseToTray = bClose; }
	bool    GetDetailsOnClick()							{ return m_bDetailsOnClick; }
	void    SetDetailsOnClick(bool bSet)				{ m_bDetailsOnClick = bSet; }
	bool    GetBugReport()								{ return prefs->BugReport; }

	bool    GetUseDownloadAddNotifier()					{ return m_bUseDownloadAddNotifier; }
	void    SetUseDownloadAddNotifier(bool bUse)		{ m_bUseDownloadAddNotifier = bUse; }
	bool    GetUseDownloadNotifier() 					{ return m_bUseDownloadNotifier; }
	void    SetUseDownloadNotifier(bool bUse)			{ m_bUseDownloadNotifier = bUse; }
	bool    GetUseChatNotifier()						{ return m_bUseChatNotifier; }
	void    SetUseChatNotifier(bool bUse)				{ m_bUseChatNotifier = bUse; }
	bool    GetUseLogNotifier()			 				{ return m_bUseLogNotifier; }
	void    SetUseLogNotifier(bool bUse) 				{ m_bUseLogNotifier = bUse; }
	bool    GetUseSoundInNotifier()		 				{ return m_bUseSoundInNotifier; }
	void    SetUseSoundInNotifier(bool bUse)			{ m_bUseSoundInNotifier = bUse; }
	bool    GetNotifierPopsEveryChatMsg()				{ return m_bNotifierPopsEveryChatMsg; }
	void    SetNotifierPopsEveryChatMsg(bool bPop)		{ m_bNotifierPopsEveryChatMsg = bPop; }
	bool	GetNotifierPopOnImportantError()			{ return m_bNotifierImportantError; }
	void	SetNotifierPopOnImportantError(bool bPop)	{ m_bNotifierImportantError = bPop; }
	CString GetNotifierWavSoundPath() 					{ return CString(m_sNotifierSoundFilePath.Get()); }
	void	SetNotifierWavSoundPath(const CString& strPath)	{ m_sNotifierSoundFilePath.Put(strPath); }

	CString	GetIRCNick()								{ return CString(m_sIRCNick.Get()); }
	void	SetIRCNick( const CString& in_nick )		{ m_sIRCNick.Put(in_nick); }
	CString	GetIRCServer()								{ return CString(m_sIRCServer.Get()); }
	void	SetIRCServer(const CString& in_serv )		{ m_sIRCServer.Put(in_serv); }
	bool	GetIRCAddTimestamp()						{ return m_bIRCAddTimestamp; }
	void	SetIRCAddTimestamp( bool flag )				{ m_bIRCAddTimestamp = flag; }
	CString	GetIRCChanNameFilter()						{ return CString(m_sIRCChanNameFilter.Get()); }
	void	SetIRCChanNameFilter(const CString& in_name){ m_sIRCChanNameFilter.Put(in_name); }
	bool	GetIRCUseChanFilter()						{ return m_bIRCUseChanFilter; }
	void	SetIRCUseChanFilter(bool flag)				{ m_bIRCUseChanFilter = flag; }
	uint16	GetIRCChannelUserFilter()					{ return m_nIRCChannelUserFilter; }
	void	SetIRCChanUserFilter(uint16 in_user)		{ m_nIRCChannelUserFilter = in_user; }
	CString	GetIrcPerformString()						{ return CString(m_sIRCPerformString.Get()); }
	void	SetIRCPerformString(const CString& in_perf)	{ m_sIRCPerformString.Put(in_perf);}
	bool	GetIrcUsePerform()							{ return m_bIRCUsePerform; }
	void	SetIrcUsePerform(bool flag)					{ m_bIRCUsePerform = flag; }
	bool	GetIRCListOnConnect()						{ return m_bIRCListOnConnect; }
	void	SetIRCListonConnect(bool flag)				{ m_bIRCListOnConnect = flag; }
	//	bool	GetIrcAcceptLinks()							{ return m_bIRCAcceptLinks; }
	//	void	SetIrcAcceptLInks(bool flag)				{ m_bIRCAcceptLinks = flag; }
	bool	GetIrcIgnoreInfoMessage()					{ return m_bIRCIgnoreInfoMessage; }
	void	SetIrcIgnoreInfoMessage(bool flag)			{ m_bIRCIgnoreInfoMessage = flag; }
	bool	GetIrcIgnoreEmuleProtoInfoMessage()			{ return m_bIRCIgnoreEmuleProtoInfoMessage; }
	void	SetIrcIgnoreEmuleProtoInfoMessage(bool flag){ m_bIRCIgnoreEmuleProtoInfoMessage = flag;}
	WORD	GetWindowsVersion();
	bool	GetStartMinimized()							{ return m_bStartMinimized; }
	void    SetStartMinimized(bool bStart)				{ m_bStartMinimized = bStart; }
	bool	GetSmartIdCheck()							{ return m_bSmartIdCheck; }
	void	SetSmartIdCheck(bool in_smartidcheck)		{ m_bSmartIdCheck = in_smartidcheck; }
	uint8	GetSmartIdState()							{ return m_nSmartIdState; }
	void	SetSmartIdState(uint8 in_smartidstate)		{ m_nSmartIdState = in_smartidstate; }

	bool	GetVerbose()						        { return m_bVerbose; }
	void 	SetVerbose(bool bEnabled)			        { m_bVerbose = bEnabled; }
	bool	GetUpdateQueueList()				        { return m_bUpdateQueueList; }
	void	SetUpdateQueueList(bool bEnabled)			{ m_bUpdateQueueList = bEnabled; }
	bool	GetManualHighPrio()					        { return m_bManualHighPrio; }
	void	SetManualHighPrio (bool bEnabled)			{ m_bManualHighPrio = bEnabled; }
	bool	ShowOverhead()						        { return m_bShowOverhead;}
	void	SetShowOverhead(bool bit)					{ m_bShowOverhead = bit;}
	bool	BackupPreview()								{ return m_bBackupPreview;}
	void	SetBackupPreview(bool bit)					{ m_bBackupPreview = bit;}

	CString	GetTxtEditor() const				        {return prefs->TxtEditor;}

	// Barry
	int8	UseFlatBar()								{ return m_nDepth3D==0; }
	int8	Get3DDepth()                                { return m_nDepth3D; }
	void	Set3DDepth(int8 iStyle)                      { m_nDepth3D = iStyle; }
//	bool	AddNewFilesPaused()                         {CSingleLock PrefLock(&m_Pref_Mutex,TRUE); return prefs->addnewfilespaused;}
	bool    AutoConnectStaticOnly()                   	{return m_bAutoConnectStaticOnly;}	
	void    SetAutoConnectStaticOnly(bool bEnabled)		{m_bAutoConnectStaticOnly = bEnabled;}	

	DWORD	GetStatsColor(int index) 					{ return m_nStatColors[index]; }
	void	SetStatsColor(int index, DWORD value) 		{ m_nStatColors[index] = value; }
	uint8	GetGraphRatio()								{ return m_nGraphRatio; }
	void	SetGraphRatio(uint8 iNum)					{ m_nGraphRatio = iNum; }
	bool	DoUseSort()									{ return m_bUseSort; }
	void    SetUseSort(bool bUseSort)					{ m_bUseSort = bUseSort; }
	bool	DoUseSrcSortCol2()							{ return m_bUseSrcSortCol2; }
	void	SetUseSrcSortCol2(bool bUseSort)			{ m_bUseSrcSortCol2 = bUseSort; }
	int8	GetServerSortCol()							{ return m_nServerSortCol; }
	void	SetServerSortCol(int8 iCol)					{ m_nServerSortCol = iCol; }
	int8	GetUploadSortCol()							{ return m_nUploadSortCol; }
	void	SetUploadSortCol(int8 iSortCol)				{ m_nUploadSortCol = iSortCol; }
	int8	GetQueueSortCol()							{ return m_nQueueSortCol; }
	void	SetQueueSortCol(int8 iSortCol)				{ m_nQueueSortCol = iSortCol; }
	int8	GetSearchSortCol()							{ return m_nSearchSortCol; }
	void	SetSearchSortCol(int8 iSortCol)				{ m_nSearchSortCol = iSortCol; }
	int8	GetIrcSortCol()								{ return m_nIRCSortCol; }
	void	SetIrcSortCol(int8 iSortCol)				{ m_nIRCSortCol = iSortCol; }
	uint16	GetFileSortCol()							{ return m_nFileSortCol; }
	void	SetFileSortCol(uint16 uCol)					{ m_nFileSortCol = uCol; }
	int8	GetDownloadSortCol()						{ return m_nDownloadSortCol; }
	void	SetDownloadSortCol(int8 iSortCol)			{ m_nDownloadSortCol = iSortCol; }
	int8	GetSrcSortCol1()							{ return m_nSrcSortCol1; }
	void	SetSrcSortCol1(int8 iSortCol)				{ m_nSrcSortCol1 = iSortCol; }
	int8	GetSrcSortCol2()							{ return m_nSrcSortCol2; }
	void	SetSrcSortCol2(int8 iSortCol)				{ m_nSrcSortCol2 = iSortCol; }
	
	void	SetMaxDownloadConperFive(int in)			{ m_nMaxConperFive = in; }
	uint16	GetMaxConperFive()							{ return m_nMaxConperFive; }
	uint16	GetDefaultMaxConperFive();

	//void	ResetStatsColor(int index); //Cax2 changed this function to DefaultStatsColor to use it in the colour picker!
	DWORD	DefaultStatsColor(int index);
	bool	IsSafeConnectEnabled()						{ return m_bSafeConnect; }
	void	SetSafeConnectEnabled(bool in)				{ m_bSafeConnect = in; }

	//--- xrmb:partTrafficPrefs ---
	bool	DoUsePT()									{ return m_bUsePartTraffic; }
	void	SetUsePT(bool in)							{ m_bUsePartTraffic = in; }
	uint8	GetUpbarStyle()								{ return m_nUpbarPartTraffic; }
	void	SetUpbarStyle(uint8 uStyle)					{ m_nUpbarPartTraffic = uStyle; }
	uint8	GetUpbarColor()								{ return m_nUpbarColorPartTraffic; }
	void	SetUpbarColor(uint8 uColor)					{ m_nUpbarColorPartTraffic = uColor; }
	//--- :xrmb ---

	// DonGato
	bool	CommunityEnabled()							{ return m_bCommunityEnabled; }
	void	SetCommunityEnabled(bool bEnable)			{ m_bCommunityEnabled = bEnable; }
	CString	CommunityString()							{ return CString(m_sCommunityString.Get()); }
	void	SetCommunityString(const CString& strComm)	{ m_sCommunityString.Put(strComm); }
	bool	CommunityNoBanEnabled()						{ return m_bCommunityNoBan; }
	void	SetCommunityNoBanEnabled(bool bEnable)		{ m_bCommunityNoBan = bEnable; }
	uint32 	NotificationDisplayTime()					{ return m_nNotificationDisplayTime; }
	void 	SetNotificationDisplayTime(uint32 uTime)	{ m_nNotificationDisplayTime = uTime; }
	uint32 	NotificationFontSize()						{ return m_nNotificationFontSize; }
	void 	SetNotificationFontSize(uint32 uFontSize)	{ m_nNotificationFontSize = uFontSize; }
	int16 	PriorityHigh()								{ return m_nPriorityHigh; }
	void 	SetPriorityHigh(int16 iPrio)				{ m_nPriorityHigh = iPrio; }
	int16 	PriorityLow()								{ return m_nPriorityLow; }
	void 	SetPriorityLow(int16 iPrio)					{ m_nPriorityLow = iPrio; }
	bool	RestartWaiting()							{ return m_bRestartWaiting; }
	void	SetRestartWaiting(bool bSet)				{ m_bRestartWaiting = bSet; }
	bool	LogToFile()									{ return m_bLogToFile; }
	void	SetLogToFile(bool bEnable)					{ m_bLogToFile = bEnable; }
	bool	DisableXSUpTo()								{ return m_bDisableXSUpTo; }
	void	SetDisableXSUpTo(bool bDisable)				{ m_bDisableXSUpTo = bDisable; }
	uint16	XSUpTo()									{ return m_nXSUpTo; }
	void	SetXSUpTo(uint16 uVal)						{ m_nXSUpTo = uVal; }
	uint32  SrvConTimeout()								{ return m_nSrvConTimeout; }
	void	SetSrvConTimeout(uint32 timeout)			{ m_nSrvConTimeout = timeout; }
	bool	SlowComplete()								{ return m_bSlowComplete; }
	void	SetSlowComplete(uint16 bSlow)				{ m_bSlowComplete = bSlow; }
	uint16	SlowCompleteBlockSize()						{ return m_nSlowCompleteBlockSize; }
	void	SetSlowCompleteBlockSize(uint16 uSize)		{ m_nSlowCompleteBlockSize = uSize; }
	CString	GetUsedFont()								{ return CString(m_sUsedFont.Get()); }
	void	SetUsedFont(const CString& font)			{ m_sUsedFont.Put(font); }
	uint8	GetFontSize()								{ return m_nFontSize; }
	void	SetFontSize(uint8 size)						{ m_nFontSize = size; }
	enHashPriority	GetHashingPriority(void)			{ return m_HashingPriority.Get(); }
	void	SetHashingPriority(enHashPriority hashing)	{ m_HashingPriority.Put(hashing); }
	bool	IsUploadPartsEnabled()						{ return m_bUploadParts; }
	void	SetUploadPartsEnabled(bool bEnable)			{ m_bUploadParts = bEnable; }
	bool	IsA4AFStringEnabled()						{ return m_bA4AFStringEnabled; }
	void	SetA4AFStringEnabled(bool bEnable)          { m_bA4AFStringEnabled = bEnable; }
	int		GetMethod()									{ return m_nSearchMethod; }
	void	SetMethod(int Value) 						{ m_nSearchMethod = Value; }
	bool	AutoTakeED2KLinks()							{ return m_bAutoTakeLinks; }
	void	SetAutoTakeED2KLinks(bool bEnable)          { m_bAutoTakeLinks = bEnable; }
	uint16	GetFileBufferSize()							{ return m_nFileBufferSize; }
	void	SetFileBufferSize(int Value)      		    { m_nFileBufferSize = Value; }
	int		GetMainProcessPriority()					{ return m_nMainProcessPriority; }
	void	SetMainProcessPriority(int Value);

	bool	IsSCHEnabled()								{ return m_bSCHEnabled; }
	void	SetSCHEnabled(bool bEnable)					{ m_bSCHEnabled = bEnable; }
	bool	IsSCHExceptMon()							{ return m_bSCHExceptMon; }
	void	SetSCHExceptMon(bool bEnable)				{ m_bSCHExceptMon = bEnable; }
	bool	IsSCHExceptTue()							{ return m_bSCHExceptTue; }
	void	SetSCHExceptTue(bool bEnable)				{ m_bSCHExceptTue = bEnable; }
	bool	IsSCHExceptWed()							{ return m_bSCHExceptWed; }
	void	SetSCHExceptWed(bool bEnable)				{ m_bSCHExceptWed = bEnable; }
	bool	IsSCHExceptThu()							{ return m_bSCHExceptThu; }
	void	SetSCHExceptThu(bool bEnable)				{ m_bSCHExceptThu = bEnable; }
	bool	IsSCHExceptFri()							{ return m_bSCHExceptFri; }
	void	SetSCHExceptFri(bool bEnable)				{ m_bSCHExceptFri = bEnable; }
	bool	IsSCHExceptSat()							{ return m_bSCHExceptSat; }
	void	SetSCHExceptSat(bool bEnable)				{ m_bSCHExceptSat = bEnable; }
	bool	IsSCHExceptSun()							{ return m_bSCHExceptSun; }
	void	SetSCHExceptSun(bool bEnable)				{ m_bSCHExceptSun = bEnable; }
	uint32	GetSCHShift1()								{ return m_nSCHShift1; }
	void	SetSCHShift1(uint32 Time)					{ m_nSCHShift1 = Time; }
	uint32	GetSCHShift2()								{ return m_nSCHShift2; }
	void	SetSCHShift2(uint32 Time)					{ m_nSCHShift2 = Time; }
	uint16	GetSCHShift1Upload()						{ return m_nSCHShift1Upload; }
	void	SetSCHShift1Upload(uint16 Value)			{ m_nSCHShift1Upload = Value; }
	uint16	GetSCHShift1Download()						{ return m_nSCHShift1Download; }
	void	SetSCHShift1Download(uint16 Value)			{ m_nSCHShift1Download = Value; }
	uint16	GetSCHShift1conn()							{ return m_nSCHShift1conn; }
	void	SetSCHShift1conn(uint16 Value)				{ m_nSCHShift1conn = Value; }
	uint16	GetSCHShift15sec()							{ return m_nSCHShift15sec; }
	void	SetSCHShift15sec(uint16 Value)				{ m_nSCHShift15sec = Value; }
	uint16	GetSCHShift2Upload()						{ return m_nSCHShift2Upload; }
	void	SetSCHShift2Upload(uint16 Value)			{ m_nSCHShift2Upload = Value; }
	uint16	GetSCHShift2Download()						{ return m_nSCHShift2Download; }
	void	SetSCHShift2Download(uint16 Value)			{ m_nSCHShift2Download = Value; }
	uint16	GetSCHShift2conn()							{ return m_nSCHShift2conn; }
	void	SetSCHShift2conn(uint16 Value)				{ m_nSCHShift2conn = Value; }
	uint16	GetSCHShift25sec()							{ return m_nSCHShift25sec; }
	void	SetSCHShift25sec(uint16 Value)				{ m_nSCHShift25sec = Value; }

	// quick-speed changer [xrmb]
	void	SetMaxUpload(uint16 nMaxUpload) 			{ m_nMaxUpload = nMaxUpload; }
	void	SetMaxDownload(uint16 nMaxDownload)			{ m_nMaxDownload = nMaxDownload; }
	uint16  GetMaxDownloadValue()						{ return m_nMaxDownload; }

	// LANCAST (moosetea) - Get/Set Properties
	bool GetLancastEnabled()							{ return m_LancastEnabled; }
	void SetLancastEnabled(bool bEnable)				{ m_LancastEnabled = bEnable; }
	bool GetLancastUseDefaultIP()						{ return m_UseDefaultIP; }
	uint32 GetLancastIP();
	uint32 GetLancastSubnet()							{ return m_LancastSubnet; }
	uint16 GetLancastPort()								{ return m_LancastPort; }

	WINDOWPLACEMENT GetEmuleWindowPlacement()			{ return prefs->EmuleWindowPlacement; }
	void SetWindowLayout(WINDOWPLACEMENT in) 			{ prefs->EmuleWindowPlacement = in; }
	static int	GetRecommendedMaxConnections();

	//Johnny-B
	bool IsWSIntruderDetectionEnabled()					{ return m_bWebIntruderDetection; }
	void SetWSIntruderDetectionEnabled(bool bEn)		{ m_bWebIntruderDetection = bEn; }
	uint32 GetWSTempDisableLogin()						{ return m_nWebTempDisableLogin; }
	void SetWSTempDisableLogin(uint32 n)				{ m_nWebTempDisableLogin = n; }
	int8 GetWSLoginAttemptsAllowed()					{ return m_nWebLoginAttemptsAllowed; }
	void SetWSLoginAttemptsAllowed(int8 n)				{ m_nWebLoginAttemptsAllowed = n; }
	//End Johnny-B

	void	LoadServerlistAddresses() ; // Tarod (read addresses.dat every time)

	// Maverick
	bool	GetMlDonkeyCM()								{ return m_bMlDonkeyCM; }
	void	SetMlDonkeyCM(bool bEnable)					{ m_bMlDonkeyCM = bEnable; }
	
	// netwolf
	bool	GetCMNotLog()								{ return m_bCMNotLog; }	
	void	SetCMNotLog(bool bEnable)					{ m_bCMNotLog = bEnable; }

	ProxySettings GetProxy() const                      { ProxySettings Ret = prefs->proxy; return Ret; }
	void SetProxySettings(ProxySettings proxysettings)  { prefs->proxy = proxysettings; }

	CStringList shareddir_list;
	CStringList tempdir_list;	// InterCeptor (multiple tempdirs)
	CStringList adresses_list;

	void	SetLanguage();
	int		GetIPFilterLevel()							{ return 128; }
	CString GetMessageFilter()							{ return prefs->messageFilter; }
	void	SetMessageFilter(const CString& in)			{ prefs->messageFilter=in; }
	bool	ShowRatesOnTitle()							{ return prefs->showRatesInTitle; }
	void	SetRatesOnTitle(bool show) 					{ prefs->showRatesInTitle=show; }

	//START enkeyDEV(kei-kun) -Messaging preferences-
	CString   GetAwayStateMessage()						{ return CString(m_sAwayStateMessage.Get()); }
	void	SetAwayStateMessage(const CString& in)		{ m_sAwayStateMessage.Put(in); }
	uint8   GetAcceptMessagesFrom()						{ return m_nAcceptMessagesFrom; }
	void	SetAcceptMessagesFrom(uint8 in)				{ m_nAcceptMessagesFrom = in; }
	bool    GetAwayState()								{ return m_nAwayState; }
	void	SetAwayState(bool in)						{ m_nAwayState = in; }
	//END enkeyDEV(kei-kun) -Messaging preferences-

	uint16	GetMsgSessionsMax()						{ return prefs->maxmsgsessions;}

	void	SetUseProxy(bool in)		{ prefs->proxy.UseProxy=in;}
	bool	IsProxyASCWOP()				{ return m_bIsASCWOP;}
	void	SetProxyASCWOP(bool in)		{ m_bIsASCWOP=in;}

	//SyruS: cleanup
	CString GetFilenameCleanups()				{ return CString(m_sFilenameCleanups.Get()); }
	void	SetFilenameCleanups(const CString& in)		{ m_sFilenameCleanups.Put(in); }
	bool	GetAutoFilenameCleanup()			{ return m_bAutoFilenameCleanup; }
	void	SetAutoFilenameCleanup(bool in)		{ m_bAutoFilenameCleanup = in; }
	bool	GetFilenameCleanupTags()			{ return m_bFilenameCleanupTags; }
	void	SetFilenameCleanupTags(bool in)		{ m_bFilenameCleanupTags = in; }

	//(SyruS) netwolf commentfilter
	CString GetCommentFilter()							{ return CString(m_sCommentFilter.Get()); }
	void	SetCommentFilter(const CString& in)			{ m_sCommentFilter.Put(in); }

	CString	GetTemplate()							{ return CString(prefs->m_sTemplateFile);}
	void	SetTemplate(CString in)					{ sprintf(prefs->m_sTemplateFile,"%s",in);}

	// DonGato: Show Average DataRate
	bool	GetShowAverageDatarate()							{ return m_bShowAverageDatarate; }
	void	SetShowAverageDatarate(bool show) 					{ m_bShowAverageDatarate = show; }

	bool	LimitlessDownload()									{ return m_bLimitlessDownload; }
	void	SetLimitlessDownload(bool bEnable)					{ m_bLimitlessDownload = bEnable; }

protected:
	void		CreateUserHash();
	void		SetStandartValues();
private:
	CString appdir;
	Preferences_Struct_DONTUSE* prefs;
	Preferences_Ext_Struct* prefsExt;

	Preferences_Import19c_Struct* prefsImport19c;
	Preferences_Import20a_Struct* prefsImport20a;
	Preferences_Import20b_Struct* prefsImport20b;

	uchar userhash[16];
	WORD  m_wWinVer;

	void LoadPreferences();
	void SavePreferences();

private:
	CSecuredVar<CString>	m_sNick;
	WORD					m_nLanguageID;

	uint16					m_nMaxUpload;
	uint16					m_nMaxDownload;
	uint16					m_nMaxConnections;
	uint16					m_nMaxSourcePerFile;

	CSecuredVar<uint64>		m_nTotalDownloadedBytes;
	CSecuredVar<uint64>		m_nTotalUploadedBytes;

	int						m_nMaxGraphDownloadRate;
	int						m_nMaxGraphUploadRate;
	uint8					m_nGraphRatio;

	bool					m_bReconnect;
	int8					m_nDeadServer;
	int8					m_nScoreSystem;
	bool					m_bAutoServerList;
	bool					m_bAutoConnect;
	bool					m_bAutoConnectStaticOnly;
	bool					m_bAddServersFromConn;
	bool					m_bFilterBadIP;
	bool					m_bFilterServersByIP;
	CSecuredVar<enSeeShare>	m_nSeeShares;

	bool					m_bUpdateNotify;
	bool					m_bMinToTray;
	bool					m_bConfirmExit;
	bool					m_bBeepOnError;
	bool					m_bSplashScreen;
	bool					m_bOnlineSig;
	bool					m_bShowOverhead;
	bool					m_bBackupPreview;
	int8					m_nTransferDoubleclick;

	int						m_nDepth3D;
	uint16					m_nTrafficOMeterInterval;
	uint16					m_nStatsInterval;
	uint16					m_nStatsColumnWidth;
	uint16					m_nDetailColumnWidth;		//Cax2 
	CSecuredVar<CString>	m_sStatsListExpanded;

	CSecuredVar<CString>	m_sIncomingDir;
	CSecuredVar<CString>	m_sTempDir;
	CSecuredVar<CString>	m_sVlcDir;
	CSecuredVar<CString>	m_sVideoPlayer;		//Cax2 import from official client.

	uint16					m_nPort;
	uint16					m_nUDPPort;
	bool					m_bUseRQlimit;	//Cax2 change
	uint16					m_nRQlimit;

	bool					m_bA4AFStringEnabled;
	int						m_nSearchMethod;
	bool					m_bAutoTakeLinks;
	uint16					m_nFileBufferSize;
	int						m_nMainProcessPriority;

	// Scheduler
	bool					m_bSCHEnabled;
	uint32					m_nSCHShift1;
	uint32					m_nSCHShift2;
	uint16					m_nSCHShift1Upload;
	uint16					m_nSCHShift1Download;
	uint16					m_nSCHShift1conn;
	uint16					m_nSCHShift15sec;
	uint16					m_nSCHShift2Upload;
	uint16					m_nSCHShift2Download;
	uint16					m_nSCHShift2conn;
	uint16					m_nSCHShift25sec;
	bool					m_bSCHExceptMon;
	bool					m_bSCHExceptTue;
	bool					m_bSCHExceptWed;
	bool					m_bSCHExceptThu;
	bool					m_bSCHExceptFri;
	bool					m_bSCHExceptSat;
	bool					m_bSCHExceptSun;

	uint16					m_nProfile;
	bool					m_bUAP;
	bool					m_bDAP;
	bool					m_bDisableXS;
	bool					m_bStartDownloadPaused;
	bool					m_bDownloadPausedOnComplete;

	uint32					m_nBadClientBanTime;
	uint32					m_nMinRequestTime;
	uint8					m_nBadClientBan;
	bool					m_bBanMessageEnabled;
	bool					m_bBanEnabled; 	// DbT:BanEnDisable

	CSecuredVar<CString>	m_sToolbarSettings;

	bool					m_bUseProxyListenPort;
	uint16					m_nListenPort;
	bool					m_bMlDonkeyCM;
	bool					m_bCMNotLog;

	uint32					m_nSLSRestoreSourcesTime;
	uint32					m_SLSMaxSourcesPerFile;
	uint32					m_nSLSOutdated;
	bool					m_bSLSEnable;

	int16					m_nQueueSize;
	uint16					m_nDeadServerRetries;
	bool					m_bSmartIdCheck;
	uint8					m_nSmartIdState;
	bool					m_bVerbose;
	bool					m_bUpdateQueueList;
	uint16					m_nMaxConperFive;
	bool					m_bSafeConnect;

	bool					m_bMultiple;
	int8					m_nTooltipDelayTime;
	bool					m_bBringToForeground;
	int8					m_nSplitterBarPosition;
	uint16					m_nStatsMax;
	int8					m_nStatsAverageMinutes;
	bool					m_bShowToolbarSpeedMeter; 
	bool					m_bCloseToTray;
	bool					m_bDetailsOnClick;
	bool					m_bStartMinimized;
	bool					m_bCommunityEnabled;
	CSecuredVar<CString>	m_sCommunityString;
	bool					m_bCommunityNoBan;
	int16					m_nPriorityHigh;
	int16					m_nPriorityLow;
	bool					m_bRestartWaiting;
	bool					m_bLogToFile;
	bool					m_bDisableXSUpTo;
	uint16					m_nXSUpTo;
	uint32					m_nSrvConTimeout;
	bool					m_bSlowComplete;
	uint16					m_nSlowCompleteBlockSize;
	CSecuredVar<enHashPriority>	m_HashingPriority;
	bool					m_bUploadParts;

	bool					m_bUseDownloadNotifier;
	bool					m_bUseDownloadAddNotifier;
	bool					m_bUseChatNotifier;
	bool					m_bUseLogNotifier;	
	bool					m_bUseSoundInNotifier;
	bool					m_bNotifierImportantError;
	CSecuredVar<CString>	m_sNotifierSoundFilePath;
	bool					m_bNotifierPopsEveryChatMsg;
	uint32					m_nNotificationDisplayTime;
	uint32					m_nNotificationFontSize;

	CSecuredVar<CString>	m_sUsedFont;
	uint8					m_nFontSize;

	// Sort
	DWORD					m_nStatColors[15];
	bool					m_bUseSort;
	bool					m_bUseSrcSortCol2;
	int8					m_nServerSortCol;
	int8					m_nUploadSortCol;
	int8					m_nQueueSortCol;
	int8					m_nSearchSortCol;
	int8					m_nIRCSortCol;
	uint16					m_nFileSortCol;
	int8					m_nDownloadSortCol;
	int8					m_nSrcSortCol1;
	int8					m_nSrcSortCol2;

	// IRC
	CSecuredVar<CString>	m_sIRCNick;
	CSecuredVar<CString>	m_sIRCServer;
	bool					m_bIRCAddTimestamp;
	CSecuredVar<CString>	m_sIRCChanNameFilter;
	bool					m_bIRCUseChanFilter;
	uint16					m_nIRCChannelUserFilter;
	CSecuredVar<CString>	m_sIRCPerformString;
	bool					m_bIRCUsePerform;
	bool					m_bIRCListOnConnect;
	//	bool					m_bIRCAcceptLinks;
	bool					m_bIRCIgnoreInfoMessage;
	bool					m_bIRCIgnoreEmuleProtoInfoMessage;

	// Web Server
	CSecuredVar<CString>	m_sWebPassword;
	CSecuredVar<CString>	m_sLowWebPassword;
	uint16					m_nWebPort;
	bool					m_bWebEnabled;
	bool					m_bWebLowEnabled;
	bool					m_bWebUseGzip;
	int						m_nWebPageRefresh;
	//Johnny-B
	bool					m_bWebIntruderDetection;
	int8					m_nWebLoginAttemptsAllowed;
	uint8 					m_nWebTempDisableLogin;
	//End Johnny-B

	// Part Traffic
	bool					m_bUsePartTraffic;
	uint8					m_nUpbarPartTraffic;		// 0=one info 1=2 infos 2=3infos
	uint8					m_nUpbarColorPartTraffic;	// 0=rainbow style 1=more rainbow 2=downloadlist-like 3=girlie

	// LANCAST (moosetea) - Settings
	bool  					m_LancastEnabled;
	bool   					m_UseDefaultIP;
	uint32 					m_DefaultLancastIP;
	uint32					m_LancastIP;
	uint32					m_LancastSubnet;
	uint16					m_LancastPort;


	CSecuredVar<CString>	m_sToolbarBitmap;
	CSecuredVar<CString>	m_sToolbarBitmapFolder;
	int8					m_nToolbarLabels;

	//START enkeyDEV(kei-kun) -Messaging preferences-
	uint8					m_nAcceptMessagesFrom;
	int8					m_nAwayState;
	CSecuredVar<CString>	m_sAwayStateMessage;

	bool					m_bIsNTBased;

	//SyruS: cleanup
	CSecuredVar<CString>	m_sFilenameCleanups;
	bool  					m_bAutoFilenameCleanup;
	bool   					m_bFilenameCleanupTags;

	CSecuredVar<CString>	m_sCommentFilter;	//(SyruS) netwolf commentfilter

	bool					m_bDisableRatingIcons;
	bool					m_bManualHighPrio;
	bool					m_bIsASCWOP;
	static bool				bWinVerAlreadyDetected;
	bool					m_bShowAverageDatarate;
	bool					m_bLimitlessDownload;

public:
	BOOL IsNTBased(void);
};
